<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class Index
 * @package Pinpoint\V12Finance\Controller\Returned
 */

namespace Pinpoint\V12Finance\Controller\Returned;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;

class Index extends Action
{
    /**
     * @var array
     */
    const V12_APPLICATION_STATUES = [
        'R' => 'Referred',
        'D' => 'Declined',
        'A' => 'Accepted',
        'M' => 'Amended',
        'S' => 'Accepted',
        'F' => 'Funded',
        'C' => 'Cancelled'
    ];

    /**
     * @type string
     */
    const V12_ACCEPTED_STATUS = "accepted";

    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Pinpoint\V12Finance\Logger\Logger
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $_request;

    /**
     * @var \Pinpoint\V12Finance\Helper\Config
     */
    protected $configHelper;

    /**
     * @var \Pinpoint\V12Finance\Helper\EmailSender
     */
    protected $emailSender;

    /**
     * @var \Magento\Sales\Model\Order\Config
     */
    protected $orderConfig;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * Index constructor.
     * @param Context $context
     * @param PageFactory $resultPageFactory
     * @param \Magento\Checkout\Model\Cart $cart
     * @param \Pinpoint\V12Finance\Logger\Logger $logger
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Pinpoint\V12Finance\Helper\Config $configHelper
     * @param \Pinpoint\V12Finance\Helper\EmailSender $emailSender
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $collectionFactory
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        \Magento\Checkout\Model\Cart $cart,
        \Pinpoint\V12Finance\Logger\Logger $logger,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Pinpoint\V12Finance\Helper\Config $configHelper,
        \Pinpoint\V12Finance\Helper\EmailSender $emailSender,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $collectionFactory,
        \Magento\Sales\Model\Order\Config $orderConfig
    )
    {
        parent::__construct($context);

        $this->resultPageFactory = $resultPageFactory;
        $this->_cart = $cart;
        $this->_logger = $logger;
        $this->_request = $request;
        $this->_checkoutSession = $checkoutSession;
        $this->configHelper = $configHelper;
        $this->emailSender = $emailSender;
        $this->collectionFactory = $collectionFactory;
        $this->orderConfig = $orderConfig;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {

        if (!$this->validateRequest()) {
            return $this->_redirect('');
        }

        $request = $this->getRequest();

        if (is_null($this->_checkoutSession)) {

            $v12ApplicationId = $request->getParam('REF');

            /** @var \Magento\Sales\Model\Order $order */
            $order = $this->getV12Orders($v12ApplicationId);

            $this->_checkoutSession->setLastRealOrder($order);

        }

        if (!is_null($this->_checkoutSession)) {

            $order = $this->_checkoutSession->getLastRealOrder();

            $payment = $this->_checkoutSession->getLastRealOrder()->getPayment();

            $v12Information = $payment->getAdditionalInformation();

            if ((int)$v12Information['V12ApplicationId'] === (int)$request->getParam('REF')) {
                try {
                    $payment->setAdditionalInformation([
                        'SalesReference' => $request->getParam('REF'),
                        'V12ApplicationId' => $request->getParam('REF'),
                        'V12AuthCode' => $request->getParam('Auth'),
                        'v12finance_application_status' => $this->getStatus($request->getParam('Status'))
                    ])->save();
                } catch (\Exception $e) {
                    $this->_logger->error('Unable to update v12 payment information.');
                }

                try {
                    $status = $this->configHelper->getOrderStatus($this->getStatus($request->getParam('Status')));

                    if ( $status ) {
                        $this->_checkoutSession->getLastRealOrder()
                            ->setState($status)
                            ->setStatus($status)
                            ->save();
                    }


                    $v12ResponseStatus = strtolower($this->getStatus($request->getParam('Status')));

                    if ($v12ResponseStatus === self::V12_ACCEPTED_STATUS) {

                        $this->emailSender->sendEmail($order);
                    }

                } catch (\Exception $e) {
                    $this->_logger->error('Unable to update order status & state.');
                }

                if ($request->getParam('Status') === 'C') {
                    $this->messageManager->addErrorMessage(__('Your V12 request has been cancelled.'));
                    return $this->_redirect('');
                }

                if ($request->getParam('Status') === 'D') {
                    return $this->_redirect('');
                }


                return $this->_redirect('checkout/onepage/success');
            } else {
                return $this->_redirect('');
            }
        }

    }

    /**
     * @param $code
     * @return bool|mixed
     */
    protected function getStatus($code)
    {
        return isset(self::V12_APPLICATION_STATUES[$code]) ? self::V12_APPLICATION_STATUES[$code] : false;
    }

    /**
     * @return bool
     */
    protected function validateRequest()
    {
        $request = $this->getRequest();

        return $request->getParam('REF')
            && $request->getParam('SR')
            && $request->getParam('Flag')
            && $request->getParam('Status');
    }

    /**
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function getV12Orders($v12ApplicationId)
    {
        $orderCollection = $this->collectionFactory->create();
        $statuses = $this->orderConfig->getVisibleOnFrontStatuses();

        $orders = $orderCollection->addFieldToSelect(
            '*'
        );

        $orders->getSelect()
            ->join(
                ["sop" => "sales_order_payment"],
                'main_table.entity_id = sop.parent_id',
                ['additional_information', 'method']
            )
            ->where('sop.method = ? AND sop.additional_information LIKE ?', [\Pinpoint\V12Finance\Model\Payment\Method\V12Finance::CODE, "%". $v12ApplicationId. "%" ]);

        return $orders;
    }
}
