<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class StatusUpdate
 * @package Pinpoint\V12Finance\Cron
 */

namespace Pinpoint\V12Finance\Cron;

class StatusUpdate
{
    /**
     * @var array
     */
    const ORDER_STATUS = [
        '0' => 'error',
        '1' => 'incomplete',
        '2' => 'referred',
        '3' => 'declined',
        '4' => 'accepted',
        '5' => 'fulfilment',
        '100' => 'cancelled',
    ];

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var \Magento\Sales\Model\Order\Config
     */
    protected $orderConfig;

    /**
     * @var \Pinpoint\V12Finance\Model\Rest\Api\CheckApplicationStatusRequest
     */
    protected $checkApplicationStatusRequest;

    /**
     * @var \Pinpoint\V12Finance\Helper\Config
     */
    protected $configHelper;

    /**
     * @var \Pinpoint\V12Finance\Helper\EmailSender
     */
    protected $emailSender;

    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    protected $orderRepository;

    /**
     * StatusUpdate constructor.
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Pinpoint\V12Finance\Model\Rest\Api\CheckApplicationStatusRequest $checkApplicationStatusRequest
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $collectionFactory
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     * @param \Pinpoint\V12Finance\Helper\EmailSender $emailSender
     * @param \Pinpoint\V12Finance\Helper\Config $configHelper
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Pinpoint\V12Finance\Model\Rest\Api\CheckApplicationStatusRequest $checkApplicationStatusRequest,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $collectionFactory,
        \Magento\Sales\Model\Order\Config $orderConfig,
        \Pinpoint\V12Finance\Helper\EmailSender $emailSender,
        \Pinpoint\V12Finance\Helper\Config $configHelper,
        \Magento\Sales\Model\OrderRepository $orderRepository
    ) {
        $this->logger = $logger;
        $this->collectionFactory = $collectionFactory;
        $this->orderConfig = $orderConfig;
        $this->checkApplicationStatusRequest = $checkApplicationStatusRequest;
        $this->configHelper = $configHelper;
        $this->emailSender = $emailSender;
        $this->orderRepository = $orderRepository;
    }

    /**
     * Execute the cron
     *
     * @return void
     */
    public function execute()
    {
        $orders = $this->getV12Orders();

        /** @var \Magento\Sales\Model\Order $order */
        foreach ($orders as $order) {
            $v12Information = $order->getPayment()->getAdditionalInformation();
            $response = $this->checkApplicationStatusRequest->makeApplicationStatusRequest($v12Information['V12ApplicationId']);

            if (isset(self::ORDER_STATUS[(string)$response['Status']])) {
                $orderStatus = $this->configHelper->getOrderStatus(self::ORDER_STATUS[(string)$response['Status']]);

                $order->setState($orderStatus)->setStatus($orderStatus);
                try {
                    $payment = $order->getPayment();
                    $additionInformation = $payment->getAdditionalInformation();
                    $additionInformation['v12finance_application_status'] = $orderStatus;
                    $payment->setAdditionalInformation($additionInformation)->save();

                    $this->orderRepository->save($order);

                    if (self::ORDER_STATUS[(string)$response['Status']] == '4') {
                        $this->emailSender->sendEmail($order);
                    }
                } catch (\Exception $exception) {
                    $this->logger->error('Unable to save order on v12 status update: ' . $order->getIncrementId());
                }
            }
        }
    }

    /**
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function getV12Orders()
    {
        $orderCollection = $this->collectionFactory->create();
        $statuses = $this->orderConfig->getVisibleOnFrontStatuses();

        $orders = $orderCollection->addFieldToSelect(
            '*'
        )->addFieldToFilter(
            'status',
            ['in' => array_diff($statuses, ['complete', 'closed', 'canceled'])]
        );

        $orders->getSelect()
            ->join(
                ["sop" => "sales_order_payment"],
                'main_table.entity_id = sop.parent_id',
                ['method']
            )
            ->where('sop.method = ?', \Pinpoint\V12Finance\Model\Payment\Method\V12Finance::CODE);

        $orders->setOrder(
            'created_at',
            'desc'
        );

        return $orders;
    }
}
