<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class ConfigProvider
 * @package Pinpoint\V12Finance\Model
 */

namespace Pinpoint\V12Finance\Model;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Directory\Model\Currency;
use Pinpoint\V12Finance\Helper\Data as v12Helper;
use Pinpoint\V12Finance\Model\ResourceModel\Product\CollectionFactory;

class ConfigProvider implements ConfigProviderInterface
{

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Quote\Model\Quote
     */
    protected $_quote;

    /**
     * System config min deposit value
     * @var integer
     */
    protected $_minDepositPc;

    /**
     * System config max deposit value
     * @var integer
     */
    protected $_maxDepositPc;

    /**
     * Min/max deposit values based on min/max pc limits and cart total
     * @var array
     */
    protected $_depositLimits;

    /**
     * @var \Pinpoint\V12Finance\Helper\Data
     */
    protected $_v12Helper;

    /**
     * V12ConfigProvider constructor.
     * @param CheckoutSession $checkoutSession
     * @param \Magento\Directory\Model\Currency $currency
     * @param CollectionFactory $financeProduct
     * @param v12Helper $v12Helper
     */
    public function __construct(
        CheckoutSession $checkoutSession,
        Currency $currency,
        CollectionFactory $financeProduct,
        v12Helper $v12Helper
    ) {
        $this->_checkoutSession = $checkoutSession;
        $this->_currency = $currency;
        $this->_v12Helper = $v12Helper;
        $this->_financeProduct = $financeProduct;
    }

    /**
     * Get finance products data, and other data for the frontend payment form/logic
     *
     * @return array
     */
    public function getConfig()
    {
        $this->_quote          = $this->_checkoutSession->getQuote();
        $this->_minDepositPc   = $this->_v12Helper->getMinDepositPercentage();
        $this->_maxDepositPc   = $this->_v12Helper->getMaxDepositPercentage();
        $this->_getDepositLimits();

        return [
            'payment' => [
                'v12form' => [
                    'code'                    => 'ppdv12finance',
                    'cartTotal'               => number_format($this->_quote->getGrandTotal(), 2),
                    'currencySymbol'          => $this->_currency->getCurrencySymbol(),
                    'minDepositPc'            => $this->_minDepositPc,
                    'maxDepositPc'            => $this->_maxDepositPc,
                    'minDeposit'              => $this->_depositLimits[0],
                    'maxDeposit'              => $this->_depositLimits[1],
                    'variableDeposit'         => $this->_v12Helper->getVariableDeposit(),
                    'availableProducts'       => $this->getAvailableProducts()
                ]
            ]
        ];
    }

    /**
     * If min/max deposit pc set in config, set the min/max allowed deposit based on cart total
     * Include this info in the available products data
     */
    protected function _getDepositLimits()
    {
        $this->_depositLimits = [0,0];
        $cartTotal = $this->_quote->getGrandTotal();

        if ($this->_minDepositPc) {
            $this->_depositLimits[0] = number_format(($cartTotal/100) * $this->_minDepositPc, 2);
        }

        if ($this->_maxDepositPc) {
            $this->_depositLimits[1] = number_format(($cartTotal/100) * $this->_maxDepositPc, 2);
        }
    }

    /**
     * Get available/enabled V12 finance products
     *
     * @return array
     */
    protected function getAvailableProducts()
    {
        $products = [];
        //$shippingAmount = $this->_quote->getShippingAddress()->getShippingAmount();
        $grandTotal = $this->_quote->getGrandTotal();
        $cartTotal = (float) $grandTotal;

        $selectedProduct = $this->_v12Helper->getSelectedProducts();

        $productCollection = $this->_financeProduct->create();
        $productCollection->addFieldToFilter('product_id',['in' => $selectedProduct]);
        $productCollection->addActiveFilter();

        foreach ($productCollection as $_product) {

            //Finance products can have a min/max loan value,
            //check those values to ensure the product is applicable to the current cart total
            if ($_product->getMinLoan() > $cartTotal || ($_product->getMaxLoan() && $_product->getMaxLoan() < $cartTotal)) {
                continue;
            }

            //Calculate finance variables
            $interest       = $this->_getInterest($_product->getApr(), $_product->getMonths(), $cartTotal, $this->_depositLimits[0]);
            $monthlyPayment = $this->_getMonthlyPayment($_product->getApr(), $_product->getMonths(), $cartTotal, $this->_depositLimits[0]);
            $totalPayable   = number_format($cartTotal + $interest, 2);
            $loanAmount     = $cartTotal - $this->_depositLimits[0];

            //V12 provide a description string per product that can be displayed with the selected finance product
            //That description has value placeholders within it that need replacing
            $updated_description = $_product->getDescription();
            $updated_description =  str_replace('$LOAN$', $loanAmount, $updated_description);
            $updated_description =  str_replace('$INSTALMENT$', $monthlyPayment, $updated_description);
            $updated_description =  str_replace('$TERM$', $_product->getMonths(), $updated_description);
            $updated_description =  str_replace('$TOTAL_PAYABLE$', $totalPayable, $updated_description);
            $updated_description =  str_replace('$CHARGE_FOR_CREDIT$', $interest, $updated_description);
            $updated_description =  str_replace('$APR$', $_product->getApr(), $updated_description);

            $products[] = [
                'id'         => $_product->getProductId(),
                'name'       => $_product->getName(),
                'months'     => $_product->getMonths(),
                'apr'        => $_product->getApr(),
                'calcFactor' => $_product->getCalcFactor(),
                'minclean'   => '',
                'maxclean'   => '',
                'deposit'    => $this->_currency->getCurrencySymbol() . $this->_depositLimits[0],
                'updated_description'=> $updated_description,
                'description'=> $_product->getDescription(),
                'totals'     => [
                    'monthlyPayment' => $this->_currency->getCurrencySymbol() . $monthlyPayment,
                    'interest'       => $this->_currency->getCurrencySymbol() . $interest,
                    'totalPayable'   => $this->_currency->getCurrencySymbol() . $totalPayable,
                ],
                'deposit_limits'    => [
                    'min'            => $this->_depositLimits[0],
                    'max'            => $this->_depositLimits[1]
                ]
            ];
        }

        return $products;
    }

    /**
     * Get monthly payment amount based on cart total, deposit, and finance product apr/months
     *
     * @param $apr
     * @param $mnths
     * @param $total
     * @param $deposit
     * @return string
     */
    protected function _getMonthlyPayment($apr, $mnths, $total, $deposit)
    {
        $interest = $this->_getInterest($apr, $mnths, $total, $deposit);
        return number_format(($total - $deposit + $interest) / $mnths, 2);
    }

    /**
     * Get the interest amount based on cart total, min deposit, and finance product apr/months
     *
     * @param $apr
     * @param $mnths
     * @param $total
     * @param $deposit
     * @return float
     */
    protected function _getInterest($apr, $mnths, $total, $deposit)
    {
        return number_format((($total - $deposit) / 100) * $apr, 2);
    }
}
