<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class V12Finance
 * @package Pinpoint\V12Finance\Model\Payment\Method
 */

namespace Pinpoint\V12Finance\Model\Payment\Method;

use Magento\Checkout\Model\Session;
use Magento\Framework\DataObject;
use Magento\Framework\DataObjectFactory;
use Magento\Quote\Api\Data\PaymentInterface;
use Magento\Quote\Model\Quote\Payment;
use Pinpoint\V12Finance\Helper\Data as v12Helper;
use Pinpoint\V12Finance\Logger\Logger as V12Logger;
use Pinpoint\V12Finance\Model\ResourceModel\Product\CollectionFactory;
use Pinpoint\V12Finance\Model\Rest\Api\ApplicationRequest as AppRequest;

class V12Finance extends \Magento\Payment\Model\Method\AbstractMethod
{

    /**
     * Payment method code
     *
     * @var string
     */
    const CODE = 'ppdv12finance';

    /**
     * Default app processing err message
     *
     * @var string
     */
    const DEFAULT_ERROR_MESSAGE = 'There was an error processing your application.  Please try again later.';

    /**
     * Payment method code
     *
     * @var string
     */
    protected $_code = self::CODE;

    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_isGateway = true;

    /**
     * @var string
     */
    protected $_formBlockType = \Pinpoint\V12Finance\Block\Form::class;

    /**
     * @var string
     */
    protected $_infoBlockType = \Pinpoint\V12Finance\Block\Info::class;

    /**
     * @var \Pinpoint\V12Finance\Helper\Data
     */
    protected $_v12Helper;

    /**
     * @var \Magento\Framework\DataObjectFactory
     */
    protected $_objectFactory;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $_productCollection;

    /**
     * @var \Pinpoint\V12Finance\Model\ResourceModel\Product\CollectionFactory
     */
    protected $_financeProductCollection;

    /**
     * @var \Magento\Sales\Model\Order
     */
    protected $_order;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_psrlogger;

    /**
     * @var V12Logger
     */
    protected $_v12logger;

    /**
     * @var AppRequest
     */
    protected $_applicationRequest;

    /**
     * @var Session
     */
    protected $_checkoutSession;

    /**
     * V12Finance constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection
     * @param CollectionFactory $financeProductCollection
     * @param v12Helper $v12Helper
     * @param AppRequest $applicationRequest
     * @param V12Logger $v12logger
     * @param \Psr\Log\LoggerInterface $psrlogger
     * @param Session $checkoutSession
     * @param DataObjectFactory $objectFactory
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection,
        CollectionFactory $financeProductCollection,
        v12Helper $v12Helper,
        AppRequest $applicationRequest,
        V12Logger $v12logger,
        \Psr\Log\LoggerInterface $psrlogger,
        Session $checkoutSession,
        DataObjectFactory $objectFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );

        $this->_productCollection = $productCollection;
        $this->_financeProductCollection = $financeProductCollection;
        $this->_v12Helper = $v12Helper;
        $this->_objectFactory = $objectFactory;
        $this->_checkoutSession = $checkoutSession;
        $this->_applicationRequest = $applicationRequest;
        $this->_v12logger = $v12logger;
        $this->_psrlogger = $psrlogger;
    }

    /**
     * Check whether payment method can be used
     *
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     * @deprecated 100.2.0
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {

        //Check helper to see if method enabled in config
        if (!$this->_v12Helper->isEnabled() || $this->_v12Helper->isAdmin()) {
            return false;
        }

        $checkResult = $this->_objectFactory->create();
        $allowed = true;

        //Check if allowed for all products.
        if (!$this->_v12Helper->enabledAllProducts()) {
            $allowed = false;

            //If not, check quote products to see if any applicable products are available
            if ($quote) {
                $cartTotal = $quote->getGrandTotal();

                $itemIds = [];
                $items = $quote->getItemsCollection();
                foreach ($items as $item) {
                    $itemIds[] = $item->getProductId();
                }

                $allowedProducts = $this->_productCollection->create();
                $allowedProducts->addAttributeToSelect('*');
                $allowedProducts->addAttributeToFilter('v12finance_enabled', 1);
                $allowedProducts->addAttributeToFilter('entity_id', ['in'=>$itemIds]);

                //Only allow finance if there are only finance applicable products in the cart
                //We cannot allow orders that would require two different payment method types
                if ($allowedProducts->getSize() === count($itemIds)) {

                    //Check the cart total against all active finance products min/max loan values
                    //to ensure there are qualifying finance products available to display
                    $financeProducts = $this->_financeProductCollection->create();
                    $financeProducts->addFieldToSelect('*');
                    $financeProducts->addActiveFilter();
                    $financeProducts->addFieldToFilter('min_loan', ['lteq'=>$cartTotal]);

                    //@todo fix this so dont have to loop through products
                    //$financeProducts->addFieldToFilter('max_loan',['from'=>0,'to'=>$cartTotal]);
                    $availableProducts = 0;
                    foreach ($financeProducts as $financeProduct) {
                        if ($financeProduct->getMaxLoan() == 0 || $financeProduct->getMaxLoan() >= $cartTotal) {
                            $availableProducts++;
                        }
                    }

                    if ($availableProducts) {
                        $allowed = true;
                    }
                }
            }
        }

        if ($quote && $allowed) {
            $allowed = (float) $quote->getGrandTotal() >= (float) $this->_scopeConfig->getValue(\Pinpoint\V12Finance\Helper\Data::CONFIG_PATH_MIN_ORDER);
        }

        //Allow method to display?
        $checkResult->setData('is_available', $allowed);

        // for future use in observers
        $this->_eventManager->dispatch(
            'payment_method_is_active',
            [
                'result' => $checkResult,
                'method_instance' => $this,
                'quote' => $quote
            ]
        );

        return $checkResult->getData('is_available');
    }

    /**
     * Get the redirect URL.
     *
     * @return string
     */
    public function getOrderPlaceRedirectUrl()
    {
        return $this->_checkoutSession->getV12RedirectUrl();
    }

    /**
     * @return \Magento\Sales\Model\Order
     */
    private function _getOrder()
    {
        if (!$this->_order) {
            try {
                $this->_order = $this->getInfoInstance()->getOrder();
            } catch (\Exception $e) {
                $this->_v12logger->error('Unable to load order.');
            }
        }

        return $this->_order;
    }

    /**
     * @return string
     */
    protected function _getDefaultErrorMessage()
    {
        return __(self::DEFAULT_ERROR_MESSAGE);
    }

    /**
     * Assign data to info model instance, when the customer selects options on the frontend form
     *
     * @param \Magento\Framework\DataObject|mixed $data
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function assignData(\Magento\Framework\DataObject $data)
    {
        $additionalData = $data->getData(PaymentInterface::KEY_ADDITIONAL_DATA);
        if (!is_object($additionalData)) {
            $additionalData = new DataObject($additionalData ?: []);
        }

        /** @var DataObject $info */
        $info = $this->getInfoInstance();
        $info->setAdditionalInformation('v12finance_product_title', $additionalData->getProductTitle());
        $info->setAdditionalInformation('v12finance_product_id', $additionalData->getProductId());
        $info->setAdditionalInformation('v12finance_deposit_amount', $additionalData->getDepositAmount());
        $info->setAdditionalInformation('v12finance_application_status', $additionalData->getApplicationStatus());

        return $this;
    }

    /**
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @param float $amount
     * @return $this
     */
    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {

        //Make api application request, collating all required info, validating, and making call/returning response
        $requestResponse = $this->_applicationRequest->makeApplicationRequest($this->_getOrder());

        //Set v12 redirect url...to redirect customer to V12 app portal
        $this->_checkoutSession->setV12RedirectUrl($requestResponse['ApplicationFormUrl']);

        //Save application related data to payment object
        //@todo auth code is empty, update this when customer returned from portal
        //@todo $requestResponse['Status'] = 1, what does this mean!?? Status are suppossed to be descriptive strings
        $payment->setAdditionalInformation("SalesReference", $requestResponse['SalesReference']);
        $payment->setAdditionalInformation("V12AuthCode", $requestResponse['AuthorisationCode']);
        $payment->setAdditionalInformation("V12ApplicationId", $requestResponse['ApplicationId']);
        $payment->setAdditionalInformation("V12ApplicationStatus", $requestResponse['Status']);

        return $this;
    }

    /**
     * @todo
     *
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @param float $amount
     * @return $this
     */
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {

        //Make api application request, collating all required info, validating, and making call/returning response
        $requestResponse = $this->_applicationRequest->makeApplicationRequest($this->_getOrder());

        //Set v12 redirect url...to redirect customer to V12 app portal
        $this->_checkoutSession->setV12RedirectUrl($requestResponse['ApplicationFormUrl']);

        //Save application related data to payment object
        //@todo auth code is empty, update this when customer returned from portal
        //@todo $requestResponse['Status'] = 1, what does this mean!?? Status are suppossed to be descriptive strings
        $payment->setAdditionalInformation("SalesReference", $requestResponse['SalesReference']);
        $payment->setAdditionalInformation("V12AuthCode", $requestResponse['AuthorisationCode']);
        $payment->setAdditionalInformation("V12ApplicationId", $requestResponse['ApplicationId']);
        $payment->setAdditionalInformation("V12ApplicationStatus", $requestResponse['Status']);

        return $this;
    }

    /**
     * Validate payment method information object
     *
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function validate()
    {
        /*
         * calling parent validate function
         */
        parent::validate();

        $info = $this->getInfoInstance();
        $errorMsg = false;

        $v12financeProductId = $info->getAdditionalInformation('v12finance_product_id');

        if (!$v12financeProductId) {
            $errorMsg = __('Please select a V12 Finance Option.');
        }

        if ($errorMsg) {
            throw new \Magento\Framework\Exception\LocalizedException($errorMsg);
        }

        return $this;
    }

    /*This function is defined to set the Payment Action Type that is - - Authorize - Authorize and Capture Whatever has been set under Configuration of this Payment Method in Admin Panel, that will be fetched and set for this Payment Method by passing that into getConfigPaymentAction() function. */
    /**
     * @return mixed
     */
    public function getConfigPaymentAction()
    {
        return $this->getConfigData('payment_action');
    }
}
